import pandas as pd
from vretcity import constants

def convert_events(df):
    """Converts integer events to string names

    Args:
        df ([pd.DataFrame]): loaded dataframe
    """
    df['typ'] = df['typ'].replace(constants.EVENT_CODES)
    return df


def convert_times(df):
    """Converts time columns, adds timesincestart column

    Args:
        df ([type]): [description]

    Returns:
        [type]: [description]
    """
    df["timesincestart"] = (df["cas"] - df.iloc[0,0])/1000
    return df


def convert_values(df, column):
    """[summary]

    Args:
        df ([type]): [description]

    Returns:
        [type]: [description]
    """
    df[column] = pd.to_numeric(df[column].str.replace(",", "."))
    return df


def process_log(df):
    """Processes the dataframe with the most common convertors. The dataframe is modified by reference
    so no return assignment is necessary

    Args:
        df (pandas.DataFrame): vretcity data frame as loaded by load_log

    Returns:
        [pandas.DataFrame]: processed DataFrame
    """
    df = convert_events(df)
    df = convert_times(df)
    return df


EVENT_CODES = {
    0: 'Transform',
    1: 'Input',
    2: 'Interactor',
    3: 'Interactable',
    4: 'EyeBegin',
    5: 'EyeEnd',
    6: 'RayBegin',
    7: 'RayEnd',
    8: 'TriggerBegin',
    9: 'TriggerEnd',
    10: 'Action',
    11: 'HeartRate',
    12: 'ExperimentalControl'
}


import plotly.express as px

def plot_path(data, times=[], downsample=10):
    dat = data.loc[(data['objekt'] == 'VRCamera') & (data['typ'] == 'Transform'),:]
    dat = dat.iloc[range(1, dat.shape[0], downsample)]
    fig = px.line_3d(dat, x="pozice_x", y="pozice_z", z="pozice_y")
    return fig


def plot_event_value(df, timesincestart=True):
    """Plots an evolution of the value

    Wrapper around plotly.express.Line standardized so that new events can be added

    Args:
        df (pandas.DataFrame): dataframe with a specific event as collected with get_events functions
        timesincestart (bool): Should the time be displayed as is in the data or as time since the log start
    """
    xname = "cas"
    if timesincestart: xname = "timesincestart"
    fig = px.line(df, x=df[xname], y="hodnota")
    return fig


import pandas as pd

from vretcity.preprocessor import convert_values

def get_events(df):
    """Returns significant events from the log. The output ignores transform and ray information etc.

    Args:
        df ([pd.DataFrame]): The full log already preprocessed with `process_log`
    Returns:
        [pandas.DataFrame]: Dataframe with the event information
    """
    # TODO - this should be solved with some enums or other exhaustive list
    df_events = df.loc[~df["typ"].isin(["Transform", "RayBegin", "RayEnd", "EyeBegin", "EyeEnd",
        "Score", "HeartRate"]), :]
    df_events = df_events.loc[((df_events["druhy_objekt"] == "Head") & (df_events["typ"].str.contains("Trigger"))) | 
                                (~(df_events["typ"].str.contains("Trigger"))), :]
    df_events = df_events.loc[df_events["objekt"] != "UserInElevator"]
    df_events = df_events.loc[:, ["timesincestart", "typ", "objekt", "druhy_objekt", "hodnota"]]
    return df_events
    

def get_heartrate(df):
    """Extracts heartrate information form the log

    Args:
        df ([pd.DataFrame]): The full log already preprocessed with `process_log`
    Returns:
        [pandas.DataFrame]: Dataframe with the heartrate information and necessary values
    """
    df_heartrate = df.loc[df["objekt"] == "HeartRate"]
    df_heartrate = df_heartrate[["cas", "timesincestart", "hodnota"]]
    df_heartrate = convert_values(df_heartrate, "hodnota")
    return df_heartrate



import re

def export_events(df_events, filepath, include_interactions=True):
    """[summary]

    Args:
        df_events (pandas.DataFrame): DataFrame obtained usually with the getters.get_events function
        filepath (string): Filepath to the original log
        include_interactions (bool, optional): Should interaction events be included int he output. Defaults to True.
    """
    if include_interactions:
        out_filename = re.sub("\.csv", "_events.csv", filepath)
        df = df_events
    else:
        out_filename = re.sub("\.csv", "_events_nointeract.csv", filepath)
        df = df_events.loc[(~df_events["typ"].str.contains("Interac")),:].to_csv(out_filename, index=False)
    df.to_csv(out_filename, index=False)




# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['vretcity']

package_data = \
{'': ['*']}

install_requires = \
['numpy>=1.20.0,<2.0.0', 'pandas>=1.3.3,<2.0.0', 'plotly>=5.3.1,<6.0.0']

setup_kwargs = {
    'name': 'vretcity',
    'version': '0.1.1',
    'description': 'Package to preprocess and analyse data from the vretcity project at NUDZ',
    'long_description': None,
    'author': 'Lukáš Hejtmánek',
    'author_email': 'hejtmy@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



import pandas as pd
import numpy as np

from vretcity.preprocessor import convert_values

def load_log(filepth, dec="."):
    """Loads the csv log as downlaoded from the website. No data processing is done, this
    is just a wrapper around pandas.read_table having correct setup

    Args:
        filepth ([type]): path to the valid csv.
        dec (str, optional): Depending on the system locale, it is possible that different
        machines might downlaod and save the csv with a different decimal point. This
        can override the decimal values in case you get strings where there 
        should be nu,bers. Defaults to ".".

    Returns:
        pandas.DataFrame: returns a data frame
    """
    data = pd.read_table(filepth, sep=';', header=0, decimal=dec, index_col=False)
    # Tries the opposite decimal point
    if data["pozice_x"].dtype != np.float64:
        new_dec = "."
        if dec == ".": new_dec = ","
        for column in data.columns:
            if ("pozice" in column) | ("rotace" in column):
                data = convert_values(data, column)
    return data


def is_valid(df):
    """Returns if the log is valid or not. NOT IMPLEMENTED. ALWAYS RETURNS TRUE

    Args:
        df pandas.DataFrame: dataframe loaded with the load_log function
    Returns:
        bool: is dataframe valid or not
    """
    return True